#!/usr/bin/perl -w

# my $config = { 'socket' => 'mregd://command:oir4wy2g@/tmp/mregd.service' };
# my $command_hash = { 'command' => 'StatusDomain', 'domain' => 'test.com' };
#
# NORMAL USE:
#
# my $response_hash = mreg_call( $command_hash, $config );
#
#
# my $response_hash = mreg_call( $command_raw, $config );
# my $response_raw = mreg_call_raw( $command_hash, $config );
# my $response_raw = mreg_call_raw( $command_raw, $config );
# my $response_hash = mreg_parse_response( $response_raw );
# my $command_raw = mreg_create_command( $command_hash );


use Socket;
use LWP::UserAgent;
use LWP::ConnCache;

my $ua = LWP::UserAgent->new();
$ua->agent("libmreg.pl/0.1");
my $cache = LWP::ConnCache->new();
$ua->conn_cache($cache);

sub mreg_call {

    my $command = shift;
    my $config = shift;
    
    my $response = mreg_call_raw($command, $config);
    return mreg_parse_response($response);
}

sub mreg_call_raw {

    my $command = shift;
    my $config = shift;

    $command = mreg_create_command($command) if ref $command;
    
    $command =~ s/\r\n/\n/og;
    $command =~ s/\n\r/\n/og;
    $command =~ s/\r/\n/og;
    $command =~ s/[\t ]*(\n|$)[\t ]*/\n/og;
    $command =~ s/(^|\n)\[[^\n]+(\n|$)/$1/og;
    $command =~ s/(^|\n)EOF(\n|$)/$1/og;

    my $socket = $config->{'socket'};
    return mreg_call_raw_mregd($command, $config)
        if $socket =~ /^mregd:/;
    return mreg_call_raw_http($command, $config)
        if $socket =~ /^http:/ || $socket =~ /^https:/;
    return mreg_call_raw_exec($command, $config)
        if $socket =~ /^exec:/;
    return undef;
}


sub mreg_encode_http_url {
    my @hex = qw(0 1 2 3 4 5 6 7 8 9 a b c d e f);
    $_ = shift;
    $_ = "" if !defined $_;
    my $s = "";
    while ( /([^A-Za-z0-9])/ ) {
        $s .= $`;
        my $o = ord($1);
        my $n = "%".$hex[$o/16].$hex[$o%16];
        $s .= $n;
        $_ = $';
    }
    $s .= $_;
    return $s;
}


sub mreg_call_raw_http {

    my $command = shift;
    my $config = shift;
    my $socket = $config->{'socket'};
    
    my $url = $socket;
    my $data = "s_command=".mreg_encode_http_url("[COMMAND]\n".$command);
    if ( $socket =~ /\?/ ) {
        $url = $`;
        $data .= "&$'";
    }

    my $req = HTTP::Request->new(POST => $url);
    $req->content_type('application/x-www-form-urlencoded');
    $req->content($data);
    
    my $res = $ua->request($req);
    if ($res->is_success) {
        return $res->content;
    } else {
        return undef;
    }
}

sub mreg_call_raw_mregd {

    my $command = shift;
    my $config = shift;
    
    my $socket = $config->{'socket'};
    return undef if !defined $socket;

    my $login = undef;
    my $password = undef;
    if ( $socket =~ /^mregd\:\/\/([a-z0-9]+)\:([a-z0-9]+)\@/ ) {
        $login = $1;
        $password = $2;
        $socket = $';
    }
    return undef if !defined $login;
    return undef if !defined $password;
    return undef if !defined $socket;

    if ( $socket =~ /^\// ) {
        while ( $socket =~ s/\.\./\./og ) {}
        while ( $socket =~ s/\/\//\//og ) {}
        socket( SOCK, PF_UNIX, SOCK_STREAM, 0 ) || return undef;
        connect( SOCK, sockaddr_un($socket) ) || return undef;
    }
    elsif ( $socket =~ /\:?(\d+)$/ ) {
        my $port = $1;
        my $host = $` || "localhost";
        my $proto = getprotobyname("tcp");
        socket( SOCK, PF_INET, SOCK_STREAM, $proto ) || return undef;
        connect( SOCK, sockaddr_in( $port, inet_aton( $host ) ) ) || return undef;
    }
    else {
        return undef;
    }
    
    sysread SOCK, $_, 4096;
    /login/i || return undef;
    syswrite SOCK,$login."\n";
    sysread SOCK, $_, 4096;
    /password/i || return undef;
    syswrite SOCK,$password."\n";

    my $request = "[METAREGISTRY]\n"
                 ."version=1\n\n";
    $request .= "user=$config->{user}\n" if defined $config->{'user'};
    $request .= "[COMMAND]\n";
    $request .= "$command";
    $request .= "\nEOF\n";
    $request =~ s/\n+/\n/og;
    
    syswrite SOCK,$request || return undef;
    
    my $response = "";
    my $b;
    do {
       $b = "\nEOF\n"
           if (sysread SOCK, $b, 4096) <= 0;
       $response .= $b;
       if ( $b =~ /\nEOF($|\n)/ ) {
           undef $b;
       }
    } while ( defined $b );
    $response =~ s/\n+/\n/og;

    shutdown SOCK, 2;
    close( SOCK );
    return $response;
}


sub mreg_call_raw_exec {

    my $command = shift;
    my $config = shift;
    my $socket = $config->{'socket'};
    
    return undef if $socket !~ /:/;
    my $prg = $';

    my $request = "[METAREGISTRY]\n"
                 ."version=1\n\n";
    $request .= "user=$config->{user}\n" if defined $config->{'user'};
    $request .= "[COMMAND]\n";
    $request .= "$command";
    $request .= "\nEOF\n";
    $request =~ s/\n+/\n/og;
    
    use IPC::Open2;
    my $pid = open2(\*RH, \*WH, $prg);
    syswrite WH, $request;
    my $response = "";
    while (<RH>) {
        $response .= $_;
        return $response if $response =~ /\nEOF\n/;
    }
    return $response;
}



sub mreg_parse_response {
    my $response = shift;

    return () if !defined $response;
    
    my %hash;

    foreach ( split /\n/, $response ) {
        if ( /^([^\=]*[^\t\= ])[\t ]*=[\t ]*/ ) {
            my $attr = $1;
            my $value = $';
            $value =~ s/[\t ]*$//;
            if ( $attr =~ /^property\[([^\]]*)\]/i ) {
                my $prop = uc $1;
                $prop =~ s/\s//og;
                if ( exists $hash{"PROPERTY"}->{$prop} ) {
                    push @{$hash{"PROPERTY"}->{$prop}}, $value;
                }
                else {
                     $hash{"PROPERTY"}->{$prop} = [$value];
                }
            }
            else {
                $hash{uc $attr} = $value;
            }
        }
        elsif ( /:([^\n]*)/ ) {
            my $attr = $`;
            my $value = $1;
            if ( $attr =~ /^property\[([^\]]*)\]/i ) {
                my $prop = uc $1;
                $prop =~ s/\s//og;
                if ( exists $hash{"PROPERTY"}->{$prop} ) {
                    push @{$hash{"PROPERTY"}->{$prop}}, $value;
                }
                else {
                     $hash{"PROPERTY"}->{$prop} = [$value];
                }
            }
            else {
                $hash{uc $attr} = $value;
            }
        }
    }
    return \%hash;
}


sub mreg_create_command {

    my $hash = shift;

    my $command = "";
    my $k;
    foreach $k ( keys %{$hash} ) {
        if ( defined $hash->{$k} ) {
            if ( ref $hash->{$k} ) {
                my $i;
                for ( $i = 0; $i <= $#{$hash->{$k}}; $i++ ) {
                    if ( defined $hash->{$k}->[$i] ) {
                        my $v = $hash->{$k}->[$i];
                        $v =~ s/\r|\n//og;
                        $command .= "$k$i=$v\n";
                    }
                }
            }
            else {
                my $v = $hash->{$k};
                $v =~ s/\r|\n//og;
                $command .= "$k=$v\n";
            }
        }
    }
    return $command;
}


sub mreg_create_response {

    my $hash = shift;
    return $hash if !ref $hash;
    my $response = "";
    my $k;
    foreach $k ( keys %{$hash} ) {
        if ( defined $hash->{$k} ) {
            if (uc($k) ne "PROPERTY") {
                my $v = $hash->{$k};
                $v =~ s/\r|\n//og;
                $response .= "$k=$v\n";
            }
	    else {
	        my $k2;
                foreach $k2 ( keys %{$hash->{$k}} ) {
		    if ( ref $hash->{$k}->{$k2} ) {
			my @l = @{$hash->{$k}->{$k2}};
			my $i = 0;
			foreach ( @l ) {
                            $response .= "PROPERTY[$k2][$i]=$_\n";
			    $i++;
			}
		    }
		    else {
                        $response .= "PROPERTY[$k2][0]=".$hash->{$k}->{$k2}."\n";
		    }
		}
	    }
        }
    }
    return $response;
}





1;
